<?php
require_once 'PHPUnit/Framework.php';
require_once 'Maarch_CLITools/ArgsParser.php';

class ArgsParserForTest extends ArgsParser {
    function &__get($key)
    {
        if (property_exists($this, $key)) {
            return $this->$key;
        }
        return null;
    }

    function __set($key, $value)
    {
        $this->$key = $value;
    }

    function __call($name, $arguments)
    {
        return call_user_func_array(Array($this, $name), $arguments);
    }
}


class ArgsParserTest extends PHPUnit_Framework_TestCase{
    public function setUp(){
        $this->parser = new ArgsParserForTest();
        $this->assertTrue($this->parser instanceof ArgsParser);
    }
    public function tearDown(){
        unset($this->parser);
        $this->assertObjectNotHasAttribute("parser", $this);
    }


    public function testInitialisation(){
        $this->assertObjectHasAttribute("_options", $this->parser);
        $this->assertAttributeSame(Array("positional"=>Array()), "_options", $this->parser);

        $this->assertObjectHasAttribute("_mapping", $this->parser);
        $this->assertAttributeSame(Array(), "_mapping", $this->parser);

        $this->assertObjectHasAttribute("_args", $this->parser);
        $this->assertAttributeSame(Array(), "_args", $this->parser);

        $this->assertObjectHasAttribute("_positionals", $this->parser);
        $this->assertAttributeSame(Array(), "_positionals", $this->parser);

        $this->assertObjectHasAttribute("_name", $this->parser);
        $this->assertAttributeSame("__main__", "_name", $this->parser);

    }


    /***************************************************************
     ** add_switch                                                **
     ***************************************************************/

    function testAddSwitchNoInfo(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_switch("quiet");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["default"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
    }

    function testAddSwitchOnlyShort(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_switch("quiet", Array("short" => "q"));
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["default"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertArrayHasKey('q', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["q"]);
    }

    function testAddSwitchOnlyLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_switch("quiet", Array("long" => "quiet"));
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["default"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertArrayHasKey('quiet', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["quiet"]);
    }

    function testAddSwitchShortLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_switch("quiet", Array("short" => "q", "long" => "quiet"));
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->_args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["default"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertArrayHasKey('q', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["q"]);
        $this->assertArrayHasKey('quiet', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["quiet"]);
    }

    function testAddSwitchOnlyCatalog(){
        $this->parser->add_switch("quiet", Array("default" => true, "help" => "my help"));
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame(true, $this->parser->_args["quiet"]["default"]);
        $this->assertSame("my help", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
    }

    function testAddSwitchAll(){
        $this->parser->add_switch("quiet", Array("short" => "q", "long" => "quiet", "default" => 42, "help" => "my help"));
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->_args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame(42, $this->parser->_args["quiet"]["default"]);
        $this->assertSame("my help", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(2, count($this->parser->_mapping));
    }

    function testAddSwitchTrueAsDefaultValue(){
        $this->parser->add_switch("quiet", Array("default" => true));
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame(true, $this->parser->_args["quiet"]["default"]);
    }

    function testAddSwitchNullAsDefaultValue(){
        $this->parser->add_switch("quiet", Array("default" => null));
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
    }


    /***************************************************************
     ** add_argument                                              **
     ***************************************************************/

    function testAddArgumentNoInfo(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
    }

    function testAddArgumentOnlyShort(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", Array("short" => "q"));
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertArrayHasKey('q', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["q"]);
    }

    function testAddArgumentOnlyLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", Array("long" => "quiet"));
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertArrayHasKey('quiet', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["quiet"]);
    }

    function testAddArgumentShortLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", Array("short" => "q", "long" => "quiet"));
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->_args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertArrayHasKey('q', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["q"]);
        $this->assertArrayHasKey('quiet', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["quiet"]);
    }

    function testAddArgumentOnlyCatalog(){
        $this->parser->add_arg("quiet", Array("default"=> 42, "help" => "my help"));
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(42, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("my help", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
    }

    function testAddArgumentAll(){
        $this->parser->add_arg("quiet", Array("short" => "q", "long" => "quiet", "default" => 42, "help" => "my help"));
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->_args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(42, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("my help", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(2, count($this->parser->_mapping));
    }

    function testAddArgumentMandatoryNoDefault(){
        $this->parser->add_arg("quiet", Array("mandatory" => true));
        $this->assertArrayHasKey('quiet', $this->parser->_args);

        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(true, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertArrayNotHasKey('quiet', $this->parser->_options);
    }

    function testAddArgumentMandatoryDefault(){
        $this->parser->add_arg("quiet", Array("mandatory" => true, "default" => 42));
        $this->assertArrayHasKey('quiet', $this->parser->_args);

        $this->assertSame(42, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(true, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertArrayHasKey('quiet', $this->parser->_options);
    }

    function testAddArgumentOptionalNoDefault(){
        $this->parser->add_arg("quiet", Array("mandatory" => false));
        $this->assertArrayHasKey('quiet', $this->parser->_args);

        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertArrayNotHasKey('quiet', $this->parser->_options);
    }

    function testAddArgumentOptionalDefault(){
        $this->parser->add_arg("quiet", Array("mandatory" => false, "default" => 42));
        $this->assertArrayHasKey('quiet', $this->parser->_args);

        $this->assertSame(42, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertArrayHasKey('quiet', $this->parser->_options);
    }

    function testAddArgNulleAsDefaultValue(){
        $this->parser->add_arg("quiet", Array("default" => null));
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
    }


    /***************************************************************
     ** add_positional                                            **
     ***************************************************************/

    function testAddPositionalNoInfo(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_positional("quiet");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame("positional", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
        $this->assertSame(Array("quiet"), $this->parser->_positionals);
    }


    function testAddPositionalAll(){
        $this->parser->add_positional("quiet", Array( "mandatory" => true, "default" => 42, "help" => "my help"));
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame("positional", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(42, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(true, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("my help", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
        $this->assertSame(Array("quiet"), $this->parser->_positionals);
    }


    /***************************************************************
     ** add_command                                               **
     ***************************************************************/

    function testAddCommandMinimal(){
        #add_command($name, $aliases, &$parser, $help="")
        $subparser = new ArgsParserForTest();
        $this->parser->add_command("sub", Array(), $subparser);
        $this->assertSame("sub", $subparser->name);
        $this->assertArrayHasKey('sub', $this->parser->_args);
        $this->assertSame('sub', $this->parser->_args['sub']['name']);
        $this->assertSame('command', $this->parser->_args['sub']['type']);
        $this->assertSame('', $this->parser->_args['sub']['help']);
        $this->assertSame(Array(), $this->parser->_args['sub']['aliases']);
        $this->assertSame($subparser, $this->parser->_args['sub']['parser']);
        $this->assertArrayHasKey('sub', $this->parser->_mapping);
        $this->assertSame("sub", $this->parser->_mapping["sub"]);
    }

    function testAddCommandWithAliases(){
        #add_command($name, $aliases, &$parser, $help="")
        $subparser = new ArgsParserForTest();
        $this->parser->add_command("sub", Array("sub2", "sub3"), $subparser, "my help");
        $this->assertSame("sub", $subparser->name);
        $this->assertArrayHasKey('sub', $this->parser->_args);
        $this->assertSame('sub', $this->parser->_args['sub']['name']);
        $this->assertSame('command', $this->parser->_args['sub']['type']);
        $this->assertSame("my help", $this->parser->_args['sub']['help']);
        $this->assertSame(Array("sub2", "sub3"), $this->parser->_args['sub']['aliases']);
        $this->assertSame($subparser, $this->parser->_args['sub']['parser']);
        $this->assertArrayHasKey('sub', $this->parser->_mapping);
        $this->assertSame("sub", $this->parser->_mapping["sub"]);
        $this->assertArrayHasKey('sub2', $this->parser->_mapping);
        $this->assertSame("sub", $this->parser->_mapping["sub2"]);
        $this->assertArrayHasKey('sub3', $this->parser->_mapping);
        $this->assertSame("sub", $this->parser->_mapping["sub3"]);
    }


    /***************************************************************
     ** check_mandatory                                           **
     ***************************************************************/

    function testCheckMandatoryArgDefaultNotGiven(){
        $this->parser->add_arg("quiet", Array("default" => 42, "mandatory" => true, "help" => "my help"));
        $this->assertTrue($this->parser->_check_mandatory_args());
    }

    function testCheckMandatoryArgDefaultGiven(){
        $this->parser->add_arg("quiet", Array("default" => 42, "mandatory" => true, "help" => "my help"));
        $this->parser->_options['quiet'] = '27';
        $this->assertTrue($this->parser->_check_mandatory_args());
    }

    /**
     * @expectedException Exception
     */
    function testCheckMandatoryArgNoDefaultNotGiven(){
        $this->parser->add_arg("quiet", Array("default" => null, "mandatory" => true, "help" => "my help"));
        $this->parser->_check_mandatory_args();
    }

    function testCheckMandatoryArgNoDefaultGiven(){
        $this->parser->add_arg("quiet", Array("default" => null, "mandatory" => true, "help" => "my help"));
        $this->parser->_options['quiet'] = '27';
        $this->assertTrue($this->parser->_check_mandatory_args());
    }

}